@@ -1,11 +1,15 @@
 # import signal  # used to catch control C
+import logging
+import queue
 import time
 
 from crownstone_core.protocol.BlePackets import ControlPacket
 from crownstone_core.protocol.BluenetTypes import ControlType
 
+from crownstone_uart.core.modules.ControlHandler import ControlHandler
 from crownstone_uart.core.dataFlowManagers.UartWriter import UartWriter
 from crownstone_uart.core.modules.MeshHandler import MeshHandler
+from crownstone_uart.core.modules.StateHandler import StateHandler
 
 from crownstone_uart.core.dataFlowManagers.StoneManager import StoneManager
 from crownstone_uart.core.modules.UsbDevHandler import UsbDevHandler
@@ -18,22 +22,21 @@ from crownstone_uart.core.uart.UartTypes import UartTxType, UartMessageType
 from crownstone_uart.core.uart.uartPackets.UartWrapperPacket import UartWrapperPacket
 from crownstone_uart.topics.SystemTopics import SystemTopics
 
+_LOGGER = logging.getLogger(__name__)
 
 class CrownstoneUart:
-    __version__ = "2.0.0"
+    __version__ = "2.1.0"
 
-    def __init__(self, loop=None):
+    def __init__(self):
         self.uartManager = None
         self.running = True
-        if loop is None:
-            self.loop = asyncio.get_event_loop()
-        else:
-            self.loop = loop
 
-        self.uartManager = UartManager()
-        
+        self.manager_exception_queue = queue.Queue()
+        self.uartManager = UartManager(self.manager_exception_queue)
         self.stoneManager = StoneManager()
 
+        self.control = ControlHandler()
+        self.state = StateHandler()
         self.mesh = MeshHandler()
         # only for development. Generally undocumented.
         self._usbDev = UsbDevHandler()
@@ -45,16 +48,19 @@ class CrownstoneUart:
         return self.uartManager.is_ready()
 
 
+
+
     async def initialize_usb(self, port = None, baudrate=230400, writeChunkMaxSize=0):
-        '''
-        writing in chunks solves issues writing to certain JLink chips. A max chunkSize of 64 was found to work well for our case.
-        For normal usage with Crownstones this is not required.
-        writeChunkMaxSize of 0 will not send the payload in chunks
-        :param port:
-        :param baudrate:
-        :param writeChunkMaxSize:
-        :return:
-        '''
+        """
+        Initialize a Crownstone serial device. 
+            
+        :param port: serial port of the USB. e.g. '/dev/ttyUSB0' or 'COM3'.
+        :param baudrate: baudrate that should be used for this connection. default is 230400.
+        :param writeChunkMaxSize: writing in chunks solves issues writing to certain JLink chips. A max chunkSize of 64 was found to work well for our case.
+            For normal usage with Crownstones this is not required. a writeChunkMaxSize of 0 will not send the payload in chunks.
+        
+        This method is a coroutine.
+        """
         self.uartManager.config(port, baudrate, writeChunkMaxSize)
 
         result = [False]
@@ -65,21 +71,28 @@ class CrownstoneUart:
         self.uartManager.start()
 
         while not result[0] and self.running:
+            try:
+                exception, exception_value, trace = self.manager_exception_queue.get(block=False)
+                self.uartManager.join()
+                self.stop()
+                raise exception(exception_value)
+            except queue.Empty:
+                pass
+
             await asyncio.sleep(0.1)
 
         UartEventBus.unsubscribe(event)
-        
+
 
     def initialize_usb_sync(self, port = None, baudrate=230400, writeChunkMaxSize=0):
-        '''
-                writing in chunks solves issues writing to certain JLink chips. A max chunkSize of 64 was found to work well for our case.
-                For normal usage with Crownstones this is not required.
-                writeChunkMaxSize of 0 will not send the payload in chunks
-                :param port:
-                :param baudrate:
-                :param writeChunkMaxSize:
-                :return:
-                '''
+        """
+        Initialize a Crownstone serial device. 
+            
+        :param port: serial port of the USB. e.g. '/dev/ttyUSB0' or 'COM3'.
+        :param baudrate: baudrate that should be used for this connection. default is 230400.
+        :param writeChunkMaxSize: writing in chunks solves issues writing to certain JLink chips. A max chunkSize of 64 was found to work well for our case.
+            For normal usage with Crownstones this is not required. a writeChunkMaxSize of 0 will not send the payload in chunks.
+        """
         self.uartManager.config(port, baudrate, writeChunkMaxSize)
 
         result = [False]
@@ -92,7 +105,16 @@ class CrownstoneUart:
 
         try:
             while not result[0] and self.running:
+                try:
+                    exc = self.manager_exception_queue.get(block=False)
+                    self.uartManager.join()
+                    self.stop()
+                    raise exc[0](exc[1])
+                except queue.Empty:
+                    pass
+
                 time.sleep(0.1)
+
         except KeyboardInterrupt:
             print("\nClosing Crownstone Uart.... Thanks for your time!")
             self.stop()
@@ -105,7 +127,7 @@ class CrownstoneUart:
             self.uartManager.stop()
         self.running = False
 
-    #
+
     def switch_crownstone(self, crownstoneId: int, on: bool):
         """
         :param crownstoneId:
@@ -136,13 +158,13 @@ class CrownstoneUart:
 
     def uart_echo(self, payloadString):
         # wrap that in a control packet
-        controlPacket = ControlPacket(ControlType.UART_MESSAGE).loadString(payloadString).getPacket()
+        controlPacket = ControlPacket(ControlType.UART_MESSAGE).loadString(payloadString).serialize()
 
         # wrap that in a uart message
-        uartMessage   = UartMessagePacket(UartTxType.CONTROL, controlPacket).getPacket()
+        uartMessage   = UartMessagePacket(UartTxType.CONTROL, controlPacket).serialize()
 
         # finally, wrap it in an uart wrapper packet
-        uartPacket    = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).getPacket()
+        uartPacket    = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).serialize()
 
         # send over uart
         result = UartWriter(uartPacket).write_sync()
