@@ -21,14 +21,15 @@ from crownstone_uart.topics.SystemTopics import SystemTopics
 
 class MeshHandler:
 
+    # TODO: make async, wait for uart reply.
     def turn_crownstone_on(self, crownstone_id: int):
         self._switch_crownstone(crownstone_id, SwitchValSpecial.SMART_ON)
 
-
+    # TODO: make async, wait for uart reply.
     def turn_crownstone_off(self, crownstone_id: int):
         self._switch_crownstone(crownstone_id, 0)
 
-
+    # TODO: make async, wait for uart reply.
     def set_crownstone_switch(self, crownstone_id: int, switch_val: int):
         """
         :param crownstone_id:
@@ -38,7 +39,7 @@ class MeshHandler:
 
         self._switch_crownstone(crownstone_id, switch_val)
 
-
+    # TODO: make async, wait for uart reply.
     def _switch_crownstone(self,crownstone_id: int, switch_val: int):
         """
         :param crownstone_id:
@@ -50,16 +51,16 @@ class MeshHandler:
         stoneSwitchPacket = StoneMultiSwitchPacket(crownstone_id, switch_val)
 
         # wrap it in a mesh multi switch packet
-        meshMultiSwitchPacket = MeshMultiSwitchPacket([stoneSwitchPacket]).getPacket()
+        meshMultiSwitchPacket = MeshMultiSwitchPacket([stoneSwitchPacket]).serialize()
 
         # wrap that in a control packet
-        controlPacket = ControlPacket(ControlType.MULTISWITCH).loadByteArray(meshMultiSwitchPacket).getPacket()
+        controlPacket = ControlPacket(ControlType.MULTISWITCH).loadByteArray(meshMultiSwitchPacket).serialize()
 
         # wrap that in a uart message
-        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).getPacket()
+        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).serialize()
 
         # finally wrap it in a uart wrapper packet
-        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).getPacket()
+        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).serialize()
 
         # send over uart
         UartEventBus.emit(SystemTopics.uartWriteData, uartPacket)
@@ -76,8 +77,24 @@ class MeshHandler:
 
 
     async def send_no_op(self):
-        no_op_packet = ControlPacket(ControlType.NO_OPERATION)
-        await self._command_via_mesh_broadcast(no_op_packet.getPacket())
+        control_packet = ControlPacket(ControlType.NO_OPERATION)
+        await self._command_via_mesh_broadcast(control_packet.serialize())
+
+    async def reset_rssi_between_stones(self, crownstone_uid_array: List[int] = None):
+        control_packet = ControlPacket(ControlType.RESET_RSSI_BETWEEN_STONES)
+        if (crownstone_uid_array == None):
+            return await self._command_via_mesh_broadcast(control_packet.serialize())
+        else:
+            return await self._command_via_mesh_broadcast_acked(crownstone_uid_array, control_packet.serialize())
+
+    async def reset_errors(self):
+        control_packet = ControlPacketsGenerator.getResetErrorPacket(0xFFFFFFFF)
+        await self._command_via_mesh_broadcast(control_packet)
+
+    async def set_tx_power(self, crownstone_uid_array: List[int], txPower: int):
+        statePacket = ControlStateSetPacket(StateType.TX_POWER)
+        statePacket.loadInt8(txPower)
+        return await self._command_via_mesh_broadcast_acked(crownstone_uid_array, statePacket.serialize())
 
     async def set_ibeacon_uuid(self, crownstone_id: int, uuid: str, index: int = 0) -> MeshResult:
         """
@@ -89,7 +106,7 @@ class MeshHandler:
         """
         statePacket = ControlStateSetPacket(StateType.IBEACON_UUID, index)
         statePacket.loadByteArray(Conversion.ibeaconUUIDString_to_reversed_uint8_array(uuid))
-        return await self._set_state_via_mesh_acked(crownstone_id, statePacket.getPacket())
+        return await self._set_state_via_mesh_acked(crownstone_id, statePacket.serialize())
 
 
     async def set_ibeacon_major(self, crownstone_id: int, major: int, index: int = 0) -> MeshResult:
@@ -102,7 +119,7 @@ class MeshHandler:
         """
         statePacket = ControlStateSetPacket(StateType.IBEACON_MAJOR, index)
         statePacket.loadUInt16(major)
-        return await self._set_state_via_mesh_acked(crownstone_id, statePacket.getPacket())
+        return await self._set_state_via_mesh_acked(crownstone_id, statePacket.serialize())
 
 
     async def set_ibeacon_minor(self, crownstone_id: int, minor: int, index: int = 0) -> MeshResult:
@@ -115,7 +132,7 @@ class MeshHandler:
         """
         statePacket = ControlStateSetPacket(StateType.IBEACON_MINOR, index)
         statePacket.loadUInt16(minor)
-        return await self._set_state_via_mesh_acked(crownstone_id, statePacket.getPacket())
+        return await self._set_state_via_mesh_acked(crownstone_id, statePacket.serialize())
 
 
     async def periodically_activate_ibeacon_index(self, crownstone_uid_array: List[int], index : int, interval_seconds: int, offset_seconds: int = 0) -> MeshResult:
@@ -186,10 +203,10 @@ class MeshHandler:
     async def _set_state_via_mesh_acked(self, crownstone_id: int, packet: bytearray) -> MeshResult:
         # 1:1 message to N crownstones with acks (only N = 1 supported for now)
         # flag value: 2
-        corePacket    = MeshSetStatePacket(crownstone_id, packet).getPacket()
-        controlPacket = ControlPacket(ControlType.MESH_COMMAND).loadByteArray(corePacket).getPacket()
-        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).getPacket()
-        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).getPacket()
+        corePacket    = MeshSetStatePacket(crownstone_id, packet).serialize()
+        controlPacket = ControlPacket(ControlType.MESH_COMMAND).loadByteArray(corePacket).serialize()
+        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).serialize()
+        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).serialize()
 
         resultCollector     = Collector(timeout=2,  topic=SystemTopics.resultPacket)
         individualCollector = BatchCollector(timeout=15, topic=SystemTopics.meshResultPacket)
@@ -217,10 +234,10 @@ class MeshHandler:
         # this is only for time and noop
         # broadcast to all:
         # value: 1
-        corePacket = MeshBroadcastPacket(packet).getPacket()
-        controlPacket = ControlPacket(ControlType.MESH_COMMAND).loadByteArray(corePacket).getPacket()
-        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).getPacket()
-        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).getPacket()
+        corePacket = MeshBroadcastPacket(packet).serialize()
+        controlPacket = ControlPacket(ControlType.MESH_COMMAND).loadByteArray(corePacket).serialize()
+        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).serialize()
+        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).serialize()
 
         resultCollector = Collector(timeout=2, topic=SystemTopics.resultPacket)
 
@@ -244,10 +261,10 @@ class MeshHandler:
         # this is only for the set_iBeacon_config_id
         # broadcast to all, but retry until ID's in list have acked or timeout
         # value: 3
-        corePacket    = MeshBroadcastAckedPacket(crownstone_uid_array, packet).getPacket()
-        controlPacket = ControlPacket(ControlType.MESH_COMMAND).loadByteArray(corePacket).getPacket()
-        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).getPacket()
-        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).getPacket()
+        corePacket    = MeshBroadcastAckedPacket(crownstone_uid_array, packet).serialize()
+        controlPacket = ControlPacket(ControlType.MESH_COMMAND).loadByteArray(corePacket).serialize()
+        uartMessage = UartMessagePacket(UartTxType.CONTROL, controlPacket).serialize()
+        uartPacket = UartWrapperPacket(UartMessageType.UART_MESSAGE, uartMessage).serialize()
 
         resultCollector     = Collector(timeout=2, topic=SystemTopics.resultPacket)
         individualCollector = BatchCollector(timeout=15, topic=SystemTopics.meshResultPacket)
