@@ -10,13 +10,15 @@ from crownstone_uart.core.UartEventBus import UartEventBus
 from crownstone_uart.core.uart.UartParser import UartParser
 from crownstone_uart.core.uart.UartReadBuffer import UartReadBuffer
 from crownstone_uart.topics.SystemTopics import SystemTopics
+from crownstone_uart.Exceptions import UartBridgeError, UartException
 
 _LOGGER = logging.getLogger(__name__)
 
 
 class UartBridge(threading.Thread):
 
-    def __init__(self, port, baudrate, writeChunkMaxSize=0):
+    def __init__(self, exception_queue, port, baudrate, writeChunkMaxSize=0):
+        self.bridge_exception_queue = exception_queue
         self.baudrate = baudrate
         self.port = port
         self.writeChunkMaxSize = writeChunkMaxSize
@@ -27,6 +29,7 @@ class UartBridge(threading.Thread):
         self.running = True
         self.parser = UartParser()
         self.eventId = UartEventBus.subscribe(SystemTopics.uartWriteData, self.write_to_uart)
+
         threading.Thread.__init__(self)
 
     def __del__(self):
@@ -34,8 +37,11 @@ class UartBridge(threading.Thread):
 
 
     def run(self):
-        self.start_serial()
-        self.start_reading()
+        try:
+            self.start_serial()
+            self.start_reading()
+        except (UartException, BaseException):
+            self.bridge_exception_queue.put(sys.exc_info())
 
 
     def stop(self):
@@ -53,8 +59,10 @@ class UartBridge(threading.Thread):
             self.serialController.timeout = UART_READ_TIMEOUT
             self.serialController._write_timeout = UART_WRITE_TIMEOUT
             self.serialController.open()
-        except OSError or serial.SerialException or KeyboardInterrupt:
+        except OSError or serial.SerialException or KeyboardInterrupt as serial_error:
             self.stop()
+            _LOGGER.error(serial_error)
+            raise UartException(UartBridgeError.CANNOT_OPEN_SERIAL_CONTROLLER, serial_error) from serial_error
 
 
     def start_reading(self):
@@ -108,6 +116,8 @@ class UartBridge(threading.Thread):
                 UartEventBus.emit(SystemTopics.uartWriteError, {"message":"SerialException occurred during uart write", "error": e})
             except OSError as e:
                 UartEventBus.emit(SystemTopics.uartWriteError, {"message":"OSError occurred during uart write.", "error": e})
+            except Exception as e:
+                UartEventBus.emit(SystemTopics.uartWriteError, {"message": "Unknown Exception during uart write.", "error": e})
             except:
                 e = sys.exc_info()[0]
                 UartEventBus.emit(SystemTopics.uartWriteError, {"message":"Unknown error during uart write.", "error": e})
