@@ -1,12 +1,14 @@
 import datetime
 import logging
 
-from crownstone_core.Exceptions import CrownstoneError
+from crownstone_core.Exceptions import CrownstoneException
 from crownstone_core.packets.ResultPacket import ResultPacket
 from crownstone_core.packets.ServiceData import ServiceData
 from crownstone_core.packets.serviceDataParsers.parsers import parseOpcode7
 from crownstone_core.util.Conversion import Conversion
 
+from crownstone_uart.core.uart.uartPackets.AssetMacReport import AssetMacReport
+from crownstone_uart.core.uart.uartPackets.NearestCrownstones import NearestCrownstoneTrackingUpdate, NearestCrownstoneTrackingTimeout
 from crownstone_uart.core.UartEventBus import UartEventBus
 from crownstone_uart.core.uart.UartTypes import UartRxType, UartMessageType
 from crownstone_uart.core.uart.uartPackets.AdcConfigPacket import AdcConfigPacket
@@ -48,7 +50,7 @@ class UartParser:
         :return:
         """
         if type(wrapperPacket) is not UartWrapperPacket:
-            raise TypeError
+            raise TypeError(f"Invalid type: {type(wrapperPacket)}")
 
         if wrapperPacket.protocolMajor != PROTOCOL_MAJOR:
             _LOGGER.warning(F"Unknown protocol: {wrapperPacket.protocolMajor}.{wrapperPacket.protocolMinor}")
@@ -69,11 +71,11 @@ class UartParser:
     def handleUartMessage(self, messagePacket: UartMessagePacket):
         try:
             self._handleUartMessage(messagePacket)
-        except CrownstoneError as e:
+        except CrownstoneException as e:
             # TODO: don't we catch too many errors this way
             #  For example errors from code executed by emitted events.
             #  We only want to catch parse errors, but we're too lazy to put a try catch around every parse call.
-            _LOGGER.warning(f"Parse error: {e}")
+            _LOGGER.error(f"Parse error: {e}")
 
     def _handleUartMessage(self, messagePacket: UartMessagePacket):
         """
@@ -170,6 +172,7 @@ class UartParser:
         elif opCode == UartRxType.MESH_SERVICE_DATA:
             # data type + service data (15b)
             result = parseOpcode7(messagePacket.payload)
+            _LOGGER.debug(f"Received service data: {result}")
             if hasattr(result,"crownstoneId"):
                 UartEventBus.emit(SystemTopics.stateUpdate, (result.crownstoneId, result))
 
@@ -204,6 +207,26 @@ class UartParser:
             UartEventBus.emit(UartTopics.logArray, packet)
 
 
+        #######################
+        # ASSET FILTER events #
+        #######################
+
+        elif opCode == UartRxType.ASSET_MAC_RSSI_REPORT:
+            _LOGGER.debug(f"Received ASSET_MAC_RSSI_REPORT: {messagePacket.payload}")
+            packet = AssetMacReport(messagePacket.payload)
+            UartEventBus.emit(UartTopics.assetTrackingReport, packet)
+
+        # elif opCode == UartRxType.NEAREST_CROWNSTONE_TRACKING_UPDATE:
+        #     _LOGGER.debug(f"Received NEAREST_CROWNSTONE_TRACKING_UPDATE: {messagePacket.payload}")
+        #     packet = NearestCrownstoneTrackingUpdate(messagePacket.payload)
+        #     UartEventBus.emit(UartTopics.nearestCrownstoneTrackingUpdate, packet)
+        #
+        # elif opCode == UartRxType.NEAREST_CROWNSTONE_TRACKING_TIMEOUT:
+        #     _LOGGER.debug(f"Received NEAREST_CROWNSTONE_TRACKING_TIMEOUT: {messagePacket.payload}")
+        #     packet = NearestCrownstoneTrackingTimeout(messagePacket.payload)
+        #     UartEventBus.emit(UartTopics.nearestCrownstoneTrackingTimeout, packet)
+
+
         ####################
         # Developer events #
         ####################
@@ -247,16 +270,12 @@ class UartParser:
             id = Conversion.int8_to_uint8(messagePacket.payload)
             UartEventBus.emit(DevTopics.ownCrownstoneId, id)
 
-        # elif opCode == UartRxType.MAC_ADDRESS:
-        #     if len(messagePacket.payload) == 7:
-        #         # Bug in old firmware (2.1.4 and lower) sends an extra byte.
-        #         addr = Conversion.uint8_array_to_address(messagePacket.payload[0:-1])
-        #     else:
-        #         addr = Conversion.uint8_array_to_address(messagePacket.payload)
-        #     if addr is not "":
-        #         UartEventBus.emit(DevTopics.ownMacAddress, addr)
-        #     else:
-        #         print("invalid address:", messagePacket.payload)
+        elif opCode == UartRxType.MAC_ADDRESS:
+            addr = Conversion.uint8_array_to_address(messagePacket.payload)
+            if addr != "":
+                UartEventBus.emit(DevTopics.ownMacAddress, addr)
+            else:
+                _LOGGER.warning("invalid address:", messagePacket.payload)
 
         elif opCode == UartRxType.ADC_CONFIG:
             # type is PowerCalculationsPacket
@@ -306,7 +325,7 @@ class UartParser:
             pass
 
         else:
-            _LOGGER.warning(f"Unknown opCode: {opCode}")
+            _LOGGER.debug(f"Unknown opCode: {opCode}")
 
         parsedData = None
         
